<?php
class ConnectionManager_Base
{
	/**
	 * Cached Connection objects
	 * @type Array
	 */
	protected $cache = array();

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsData;

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsIdByName = array();


	/**
	 * An array storing the correspondence between project
	 * datasource tables names and connections ids
	 * @type Array
	 */
	protected $_tablesConnectionIds;


	/**
	 * @constructor
	 */
	function __construct()
	{
		$this->_setConnectionsData();
		$this->_setTablesConnectionIds();
	}

	/**
	 * Get connection id by the table name
	 * @param String tName
	 * @return Connection
	 */
	public function getTableConnId( $tName )
	{
		return $this->_tablesConnectionIds[ $tName ];
	}


	/**
	 * Get connection object by the table name
	 * @param String tName
	 * @return Connection
	 */
	public function byTable( $tName )
	{
		$connId = $this->_tablesConnectionIds[ $tName ];
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}

	/**
	 * Get connection object by the connection name
	 * @param String connName
	 * @return Connection
	 */
	public function byName( $connName )
	{
		$connId = $this->getIdByName( $connName );
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}

	/**
	 * Get connection id by the connection name
	 * @param String connName
	 * @return String
	 */
	protected function getIdByName( $connName )
	{
		return $this->_connectionsIdByName[ $connName ];
	}

	/**
	 * Get connection object by the connection id
	 * @param String connId
	 * @return Connection
	 */
	public function byId( $connId )
	{
		if( !isset( $this->cache[ $connId ] ) ) {
			$conn = $this->getConnection( $connId );
			if( !$conn ) {
				global $restApis;
				$conn = $restApis->getConnection( $connId );
			}
			if( !$conn ) {
				$conn = $this->getDefault();
			}
			$this->cache[ $connId ] = $conn;
		}

		return $this->cache[ $connId ];
	}

	/**
	 * Get the default db connection class
	 * @return Connection
	 */
	public function getDefault()
	{
		return $this->byId( "mastation_at_localhost" );
	}

	/**
	 * Get the default db connection id
	 * @return String
	 */
	public function getDefaultConnId()
	{
		return "mastation_at_localhost";
	}



	/**
	 * Get the users table db connection
	 * @return Connection
	 */
	public function getForLogin() {
		return $this->byId( $this->getLoginConnId() );
	}

	public function getLoginConnId() {
		return "mastation_at_localhost";
	}


	/**
	 * Get the log table db connection
	 * @return Connection
	 */
	public function getForAudit()
	{
		return $this->byId( "mastation_at_localhost" );
	}

	/**
	 * Get the locking table db connection
	 * @return Connection
	 */
	public function getForLocking()
	{
		return $this->getDefault();
	}

	/**
	 * Get the 'ug_groups' table db connection
	 * @return Connection
	 */
	public function getForUserGroups() {
		return $this->byId( $this->getUserGroupsConnId() );
	}

	public function getUserGroupsConnId() {
		return "mastation_at_localhost";
	}

	/**
	 * Get the saved searches table db connection
	 * @return Connection
	 */
	public function getForSavedSearches()
	{
		return $this->byId( $this->getSavedSearchesConnId() );
	}
	
	/**
	 * Get the saved searches table db connection
	 * @return Connection
	 */
	public function getSavedSearchesConnId()
	{
		return "mastation_at_localhost";
	}	

	/**
	 * Get the webreports tables db connection
	 * @return Connection
	 */
	public function getForWebReports() 
	{
		return $this->byId( $this->getSavedSearchesConnId() );
	}

	/**
	 * Get the webreports tables db connection id
	 * @return String
	 */
	public function getWebReportsConnId() {
		return "mastation_at_localhost";
	}	
	
	/**
	 * @param String connId
	 * @return Connection
	 */
	protected function getConnection( $connId )
	{
		return false;
	}

	public function getConectionsIds()
	{
		$connectionsIds = array();
		foreach ($this->_connectionsData as $connId => $data) {
			$connectionsIds[] = $connId;
		}

		return $connectionsIds;
	}

	/**
	 * Set the data representing the project's
	 * db connection properties
	 */
	protected function _setConnectionsData()
	{
        return null;
	}

	/**
	 * Set the data representing the correspondence between
	 * the project's table names and db connections
	 */
	protected function _setTablesConnectionIds()
	{
		$connectionsIds = array();
		$connectionsIds["station"] = "mastation_at_localhost";
		$connectionsIds["carburant"] = "mastation_at_localhost";
		$connectionsIds["pompe"] = "mastation_at_localhost";
		$connectionsIds["profil"] = "mastation_at_localhost";
		$connectionsIds["cuve"] = "mastation_at_localhost";
		$connectionsIds["prixcarburant"] = "mastation_at_localhost";
		$connectionsIds["utilisateur"] = "mastation_at_localhost";
		$connectionsIds["ventepompe"] = "mastation_at_localhost";
		$connectionsIds["vue_utilisateur"] = "mastation_at_localhost";
		$connectionsIds["vue_profil"] = "mastation_at_localhost";
		$connectionsIds["lignecommande"] = "mastation_at_localhost";
		$connectionsIds["prixachat"] = "mastation_at_localhost";
		$connectionsIds["carburant_achat"] = "mastation_at_localhost";
		$connectionsIds["commande"] = "mastation_at_localhost";
		$connectionsIds["jaugecuve"] = "mastation_at_localhost";
		$connectionsIds["req_test"] = "mastation_at_localhost";
		$connectionsIds["livraisoncommande"] = "mastation_at_localhost";
		$connectionsIds["lignelivraison"] = "mastation_at_localhost";
		$connectionsIds["vue_commande"] = "mastation_at_localhost";
		$connectionsIds["last_index_pompe"] = "mastation_at_localhost";
		$connectionsIds["personnel"] = "mastation_at_localhost";
		$connectionsIds["fonction"] = "mastation_at_localhost";
		$connectionsIds["vente_du_jour"] = "mastation_at_localhost";
		$connectionsIds["etat_stock_cuve"] = "mastation_at_localhost";
		$connectionsIds["vue_station"] = "mastation_at_localhost";
		$connectionsIds["vue_carburant"] = "mastation_at_localhost";
		$connectionsIds["vue_personnel"] = "mastation_at_localhost";
		$connectionsIds["pompistes"] = "mastation_at_localhost";
		$connectionsIds["caisse"] = "mastation_at_localhost";
		$connectionsIds["recette_journaliere"] = "mastation_at_localhost";
		$connectionsIds["mouvement_caisse"] = "mastation_at_localhost";
		$connectionsIds["typemouvement"] = "mastation_at_localhost";
		$connectionsIds["bank_out"] = "mastation_at_localhost";
		$connectionsIds["typecaisse"] = "mastation_at_localhost";
		$connectionsIds["ormoney_out"] = "mastation_at_localhost";
		$connectionsIds["salaire_personnel"] = "mastation_at_localhost";
		$connectionsIds["gest_ormoney_out"] = "mastation_at_localhost";
		$connectionsIds["index_pompe"] = "mastation_at_localhost";
		$connectionsIds["transactions"] = "mastation_at_localhost";
		$connectionsIds["suivi_banque"] = "mastation_at_localhost";
		$connectionsIds["suivi_tpe"] = "mastation_at_localhost";
		$connectionsIds["suivi_ormoney"] = "mastation_at_localhost";
		$connectionsIds["suivi_creance"] = "mastation_at_localhost";
		$connectionsIds["suivi_caisse"] = "mastation_at_localhost";
		$connectionsIds["etat_vente_pompe"] = "mastation_at_localhost";
		$connectionsIds["suivi_vente_jour"] = "mastation_at_localhost";
		$connectionsIds["etat_vente_pompe_gest"] = "mastation_at_localhost";
		$connectionsIds["hisconnex"] = "mastation_at_localhost";
		$connectionsIds["personnel_sorti"] = "mastation_at_localhost";
		$connectionsIds["dernierindex"] = "mastation_at_localhost";
		$connectionsIds["suivi_depenses"] = "mastation_at_localhost";
		$connectionsIds["les_transactions"] = "mastation_at_localhost";
		$connectionsIds["recette_du_jour"] = "mastation_at_localhost";
		$connectionsIds["versement_du_jour"] = "mastation_at_localhost";
		$connectionsIds["suivi_vente_par_jour_gerant"] = "mastation_at_localhost";
		$connectionsIds["article"] = "mastation_at_localhost";
		$connectionsIds["famille"] = "mastation_at_localhost";
		$connectionsIds["codebarre"] = "mastation_at_localhost";
		$connectionsIds["article_produit"] = "mastation_at_localhost";
		$connectionsIds["article_service"] = "mastation_at_localhost";
		$connectionsIds["stocks"] = "mastation_at_localhost";
		$connectionsIds["article_all"] = "mastation_at_localhost";
		$connectionsIds["Report_stocks"] = "mastation_at_localhost";
		$connectionsIds["ventearticle"] = "mastation_at_localhost";
		$connectionsIds["ligne_vente"] = "mastation_at_localhost";
		$connectionsIds["vue_client"] = "mastation_at_localhost";
		$connectionsIds["venteservice"] = "mastation_at_localhost";
		$connectionsIds["ligneventeservice"] = "mastation_at_localhost";
		$connectionsIds["client"] = "mastation_at_localhost";
		$connectionsIds["achat"] = "mastation_at_localhost";
		$connectionsIds["choix_article_service"] = "mastation_at_localhost";
		$connectionsIds["choix_article_produit"] = "mastation_at_localhost";
		$connectionsIds["mouvement_stock"] = "mastation_at_localhost";
		$connectionsIds["fournisseur"] = "mastation_at_localhost";
		$connectionsIds["ligne_achat"] = "mastation_at_localhost";
		$connectionsIds["codebarre_choix"] = "mastation_at_localhost";
		$connectionsIds["vue_fournisseur"] = "mastation_at_localhost";
		$connectionsIds["Caisse_carburant"] = "mastation_at_localhost";
		$connectionsIds["caisse_depenses"] = "mastation_at_localhost";
		$connectionsIds["ventelubrifiants"] = "mastation_at_localhost";
		$connectionsIds["ligneventelubrifiant"] = "mastation_at_localhost";
		$connectionsIds["bank_in_versement"] = "mastation_at_localhost";
		$connectionsIds["bank_apport_perso"] = "mastation_at_localhost";
		$connectionsIds["ormoney_in"] = "mastation_at_localhost";
		$connectionsIds["maj_stock"] = "mastation_at_localhost";
		$connectionsIds["historisation"] = "mastation_at_localhost";
		$connectionsIds["req_testcaisse"] = "mastation_at_localhost";
		$connectionsIds["vente_par_pompe"] = "mastation_at_localhost";
		$connectionsIds["controle_stock_carburant"] = "mastation_at_localhost";
		$connectionsIds["tdb_controle_stock"] = "mastation_at_localhost";
		$connectionsIds["tdb_vente_par_pompe"] = "mastation_at_localhost";
		$connectionsIds["naturetransaction"] = "mastation_at_localhost";
		$connectionsIds["ecart_pompiste"] = "mastation_at_localhost";
		$connectionsIds["tpe_in"] = "mastation_at_localhost";
		$connectionsIds["creance"] = "mastation_at_localhost";
		$connectionsIds["gestion_creances"] = "mastation_at_localhost";
		$connectionsIds["credit_in"] = "mastation_at_localhost";
		$connectionsIds["creance_clients"] = "mastation_at_localhost";
		$connectionsIds["serveuremail"] = "mastation_at_localhost";
		$connectionsIds["vivo_credit"] = "mastation_at_localhost";
		$connectionsIds["vivo_debit"] = "mastation_at_localhost";
		$connectionsIds["vente_lubrifiant"] = "mastation_at_localhost";
		$connectionsIds["solde_par_jour"] = "mastation_at_localhost";
		$connectionsIds["les_soldes"] = "mastation_at_localhost";
		$connectionsIds["les_soldes_report"] = "mastation_at_localhost";
		$connectionsIds["dette_frs"] = "mastation_at_localhost";
		$connectionsIds["compte_frs"] = "mastation_at_localhost";
		$connectionsIds["le_compte_frs"] = "mastation_at_localhost";
		$connectionsIds["solde_frs_jour"] = "mastation_at_localhost";
		$connectionsIds["frs_produits"] = "mastation_at_localhost";
		$connectionsIds["frs_charges"] = "mastation_at_localhost";
		$connectionsIds["regt_frs"] = "mastation_at_localhost";
		$connectionsIds["le_solde_frs"] = "mastation_at_localhost";
		$connectionsIds["frs_autre_achat"] = "mastation_at_localhost";
		$connectionsIds["frs_autre_regt"] = "mastation_at_localhost";
		$connectionsIds["mois_vente_par_pompe"] = "mastation_at_localhost";
		$connectionsIds["jauge_du_mois"] = "mastation_at_localhost";
		$connectionsIds["vente_du_mois"] = "mastation_at_localhost";
		$connectionsIds["mois_ecart_carburant"] = "mastation_at_localhost";
		$connectionsIds["mois_tdb"] = "mastation_at_localhost";
		$connectionsIds["mois_tdb_gerant"] = "mastation_at_localhost";
		$connectionsIds["valorisation_stock"] = "mastation_at_localhost";
		$connectionsIds["date_stock"] = "mastation_at_localhost";
		$connectionsIds["valeur_stock"] = "mastation_at_localhost";
		$connectionsIds["vente_lubrifiant_gaz"] = "mastation_at_localhost";
		$connectionsIds["annee_exo"] = "mastation_at_localhost";
		$connectionsIds["exo_select"] = "mastation_at_localhost";
		$connectionsIds["exo_change"] = "mastation_at_localhost";
		$connectionsIds["Select_mois"] = "mastation_at_localhost";
		$connectionsIds["bilan"] = "mastation_at_localhost";
		$connectionsIds["date_bilan"] = "mastation_at_localhost";
		$connectionsIds["quart"] = "mastation_at_localhost";
		$connectionsIds["relevepompe"] = "mastation_at_localhost";
		$connectionsIds["db_index_pompe"] = "mastation_at_localhost";
		$connectionsIds["db_relevepompe"] = "mastation_at_localhost";
		$connectionsIds["stationsodiserv1_audit"] = "mastation_at_localhost";
		$connectionsIds["partenaire"] = "mastation_at_localhost";
		$connectionsIds["select_station"] = "mastation_at_localhost";
		$connectionsIds["bouteillegaz"] = "mastation_at_localhost";
		$connectionsIds["rapport_creance"] = "mastation_at_localhost";
		$connectionsIds["rapport_creanceclient"] = "mastation_at_localhost";
		$connectionsIds["rapport_creancepompiste"] = "mastation_at_localhost";
		$connectionsIds["select_station_change"] = "mastation_at_localhost";

		$this->_tablesConnectionIds = &$connectionsIds;
	}

	/**
	 * Check if It's possible to add to one table's sql query
	 * an sql subquery to another table.
	 * Access doesn't support subqueries from the same table as main.
	 * @param String dataSourceTName1
	 * @param String dataSourceTName2
	 * @return Boolean
	 */
	public function checkTablesSubqueriesSupport( $dataSourceTName1, $dataSourceTName2 )
	{
		$connId1 = $this->_tablesConnectionIds[ $dataSourceTName1 ];
		$connId2 = $this->_tablesConnectionIds[ $dataSourceTName2 ];

		if( $connId1 != $connId2 )
			return false;

		if( $this->_connectionsData[ $connId1 ]["dbType"] == nDATABASE_Access && $dataSourceTName1 == $dataSourceTName2 )
			return false;

		return true;
	}

	/**
	 * Close db connections
    */
	function CloseConnections()
	{
		foreach( $this->cache as $connection )
		{
			if( $connection )
				$connection->close();
		}
	}
}
?>